# -*- coding: utf8 -*-
from __future__ import unicode_literals

import random

from driverng.protocol.acknack import ACKNACKProtocol
from driverng.transport.base import TransportException
from driverng.utils import ACK, ENQ, EOT, byte2log


class LumipulseProtocol(ACKNACKProtocol):
    def send_trames(self, trame):
        """Envoie une  trame (message) sur la connexion.
        Les trames sont construites avec `build_trames` puis envoyés
        suivant le protocole automate::

            -> <enq>
            <- <ack>
            -> DATA
            <- <ack>
            -> <eot>

        :return: False si erreur sinon True (a traiter : exceptions)
        :param trames: liste de trames à envoyer
        """

        self.write(ENQ)
        try:
            data = self.read(timeout=15)
        except TransportException:
            self.logger.info("Aucun acquittement dans le timeout de 15 secondes")
            self.write(EOT)
            return False
        if not data or data[0] == ENQ:
            self.write(EOT)
            return False
        if data[0] != ACK:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<ack> expected in %s: wait %ds before re-sending <enq>",
                byte2log(data),
                self.rtimeout,
            )
            return False
        ok = super(LumipulseProtocol, self).send_trames(trame)
        self.write(EOT)
        return ok

    def recv_trames(self):
        data = self.read(timeout=1)

        if not data or data[0] != ENQ:
            self.rtimeout = random.randint(10, 20)
            self.logger.info(
                "<enq> expected in %s: wait %ds", byte2log(data), self.rtimeout
            )
            return False

        self.write(ACK)
        trame = super(LumipulseProtocol, self).recv_trames()
        self.read()  # EOT
        return trame
